import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;
import ij.text.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 22/05/02 E. Glory 	last modification 16/11/2003    

    The XYZ colour space was accepted by the CIE in 1931. It was designed to yield non-negative tri-stimulus value for each colour (Wyszecki and Stiles, 1982).
    1st component X = 0.490 R + 0.310 G + 0.200 B         
    2nd component Y = 0.177 R + 0.812 G + 0.011 B 
    3rd component Z = 0.000 R + 0.010 G + 0.990 B
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p69-71
    
    
    but we use an other matrix corresponding to illuminant D65 (sRGB)
    X = 0.412453 R + 0.357580 G + 0.180423 B         
    Y = 0.212671 R + 0.715160 G + 0.072169 B 
    Z = 0.019334 R + 0.119193 G + 0.950227 B
	   

*/


public class fromRGBtoXYZ_ implements PlugInFilter {
	ImagePlus imp;
    public  int [] Pixels;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();			
		this.Pixels=new int[h*w];		
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
	    IJ.showProgress(0.1);

	    String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));

		Pixels=(int[])ip.getPixels();
		byte[] X=new byte[this.Pixels.length];
		byte[] Y=new byte[this.Pixels.length];
		byte[] Z=new byte[this.Pixels.length];

		this.getXYZ(X,Y,Z);
		//affiche(X,Y,Z,Pixels);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);
		
		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_X",X);
		stack.addSlice(name+"_Y",Y);
		stack.addSlice(name+"_Z",Z);
		
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    	imp2.show();
    	
    	
    	

	}

    public void getXYZ(byte[] X, byte[] Y, byte[] Z) {
	    int c,r,g,b;
		float[] XYZ = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  r = (c&0xff0000)>>16;
		  g = (c&0xff00)>>8;
		  b = c&0xff;
		  
		  XYZ=RGBtoXYZ(r,g,b,XYZ);
		  
		  X[i] = (byte)((int)(XYZ[0]+0.5));
		  Y[i] = (byte)((int)XYZ[1]+0.5);
		  Z[i] = (byte)((int)XYZ[2]+0.5);		
		  
		}
	 }

     public float[] RGBtoXYZ(int r, int g, int b, float[] XYZvals){
     	 //TextWindow tw = new TextWindow("Control","img "+"level","",300,400);
     	
	       if (XYZvals==null)
    		  XYZvals =new float[3];
	       /*
	       XYZvals[0]=0.490f*r+0.310f*g+0.200f*b;       // [0;255]
	       XYZvals[1]=0.177f*r+0.812f*g+0.011f*b;       // [0;255]
	       XYZvals[2]=0.010f*g+0.990f*b;                // [0;255]
	       */
	       //tw.append("color : "+r+","+g+","+b+")");
	       XYZvals[0]=0.412453f*r + 0.357580f*g + 0.180423f*b;  // [0 ; 242.36628]
	       XYZvals[1]=0.212671f*r + 0.715160f*g + 0.072169f*b;  // [0 ; 255]
	       XYZvals[2]=0.019334f*r + 0.119193f*g + 0.950227f*b;  // [0 ; 277.63227]
	       //tw.append("avant norma "+XYZvals[0]);
	       
	       /*normalisation*/
	       XYZvals[0]= (float)(XYZvals[0]*1.052127);  // 255./242.36628 = 1,052127  [0 ; 255]
	       XYZvals[2]= (float)(XYZvals[2]*0.918481);  // 255./277.63227 = 0,918481  [0 ; 255]					
	       // tw.append("apres norma "+XYZvals[0]);
	       
	       return XYZvals;
	 }
	 
	 
	 public void affiche(byte[] X, byte[] Y, byte[] Z, int[] Pixels) {
	    TextWindow tw = new TextWindow("Control","img "+"level","",300,400); 
		for (int i=0; i<Pixels.length; i++){
			//System.out.println("pixel "+i+" : XYZ("+X[i]+","+Y[i]+","+Z[i]+")");
			int x=X[i]&0xff;
			int y=Y[i]&0xff;
			int z=Z[i]&0xff;
			tw.append("pixel "+i+" : XYZ("+x+","+y+","+z+")");
		}
	 }
}
