import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 22/05/02 E. Glory
    YCbCr is appropriate for digital coding of standard TV images (1994).
    1st component Y = 0.299 R + 0.587 G + 0.114 B         
    2nd component Cb = 0.596 R - 0.274 G - 0.322 B 
    3rd component Cr = 0.211 R - 0.523 G + 0.312 B
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p73
*/


public class fromRGBtoYCbCr_ implements PlugInFilter {
	ImagePlus imp;
    public  int [] Pixels;

  
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();		
		this.Pixels=new int[h*w];		
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
	        IJ.showProgress(0.1);

	        String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));
	
		Pixels=(int[])ip.getPixels();
		byte[] Y=new byte[this.Pixels.length];
		byte[] Cb=new byte[this.Pixels.length];
		byte[] Cr=new byte[this.Pixels.length];

		this.getYCbCr(Y,Cb,Cr);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);
		//imp2.trimProcessor();
		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_Y",Y);
		stack.addSlice(name+"_Cb",Cb);
		stack.addSlice(name+"_Cr",Cr);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    		imp2.show();

	}
       
         public void getYCbCr(byte[] Y, byte[] Cb, byte[] Cr) {
	        int c,R,G,B;
		float[] YCbCr = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  R = (c&0xff0000)>>16;
		  G = (c&0xff00)>>8;
		  B = c&0xff;
		  YCbCr=RGBtoYCbCr(R,G,B,YCbCr);
		  Y[i] = (byte)((int)(YCbCr[0]));
		  Cb[i] = (byte)((int)(YCbCr[1]+127.5f)); 
		  Cr[i] = (byte)((int)(YCbCr[2]+127.5f));
		}
	 }

         public float[] RGBtoYCbCr(int R, int G, int B, float[] YCbCrvals){
	       if (YCbCrvals==null)
    		  YCbCrvals =new float[3];

	       YCbCrvals[0]= 0.299f*R+0.587f*G+0.114f*B;       // [0;255]
	       YCbCrvals[1]=-0.169f*R-0.331f*G+0.500f*B;       // [-127.5;127.5]
 	       YCbCrvals[2]= 0.500f*R-0.418f*G-0.081f*B;       // [-127.5;127.5]
	       return YCbCrvals;
	 }
}
