import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.filter.*;

import java.util.*;
import java.awt.image.*;
import java.awt.color.*;


/** 22/05/02 E. Glory
    The XYZ colour space was accepted by the CIE in 1931. It was designed to yield non-negative tri-stimulus value for each colour (Wyszecki and Stiles, 1982).
    1st component X = 0.490 R + 0.310 G + 0.200 B         
    2nd component Y = 0.177 R + 0.812 G + 0.011 B 
    3rd component Z = 0.000 R + 0.010 G + 0.990 B
    ->The Colour Image Processsing Handbook, (Sangwine,Horne) p67-69
*/


public class fromRGBtorgb_ implements PlugInFilter {
	ImagePlus imp;
    public  int [] Pixels;
 
	public int setup(String arg, ImagePlus imp) {
	  	this.imp = imp;
		int h=imp.getHeight();
		int w=imp.getWidth();
			
		this.Pixels=new int[h*w];
		
		return DOES_RGB+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
	    IJ.showProgress(0.1);

	    String title=imp.getTitle();
		String name=title.substring(0,title.indexOf("."));
	
		Pixels=(int[])ip.getPixels();
		byte[] r=new byte[this.Pixels.length];
		byte[] g=new byte[this.Pixels.length];
		byte[] b=new byte[this.Pixels.length];

		this.getrgb(r,g,b);

		ImagePlus imp2=new ImagePlus(imp.getTitle(), ip);
		//create stack and select X channel
		ColorModel cm = LookUpTable.createGrayscaleColorModel(false);
		ImageStack stack = new ImageStack(imp.getWidth(),imp.getHeight(), cm);
		stack.addSlice(name+"_r",r);
		stack.addSlice(name+"_g",g);
		stack.addSlice(name+"_b",b);
		//System.out.println("nombre de slices "+stack.getSize());
		IJ.showProgress(1.0);
		imp2.setStack(null, stack);
    		imp2.show();

	}
       

    public void getrgb(byte[] r, byte[] g, byte[] b) {
	    int c,R,G,B;
		float[] rgb = new float[3];
		for (int i=0; i < Pixels.length; i++) {
		  c = Pixels[i];
		  R = (c&0xff0000)>>16;
		  G = (c&0xff00)>>8;
		  B = c&0xff;
		  rgb=RGBtorgb(R,G,B,rgb);
		  r[i] = (byte)((int)(rgb[0]*255.0));
		  g[i] = (byte)((int)(rgb[1]*255.0));
		  b[i] = (byte)((int)(rgb[2]*255.0));
		}
	 }


         public float[] RGBtorgb(int R, int G, int B, float[] rgbvals){
	       float sum=0.f;
	       if (rgbvals==null) rgbvals =new float[3];
    		  
	       sum=R+G+B;
	       rgbvals[0]=R/sum;
	       rgbvals[1]=G/sum;
	       rgbvals[2]=B/sum;
	       return rgbvals;
	 }

}
