import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.process.*;
import ij.gui.*;

//Domes by Gabriel Landini, G.Landini@bham.ac.uk

public class Domes_ implements PlugInFilter {
	protected boolean doIwhite;
	protected int domeheight;

	public int setup(String arg, ImagePlus imp) {

		if (arg.equals("about")){
			showAbout(); 
			return DONE;
		}

	if (imp!=null && imp.getStackSize()>1)
		{IJ.showMessage("Domes", "Stacks not supported.\nUse Domes_stack macro."); return DONE;}

		GenericDialog gd = new GenericDialog("Domes", IJ.getInstance());
		gd.addMessage("Morphological domes");
		gd.addMessage("Domes are bright regions of certain\nheight in the greyscale function.");
		gd.addNumericField ("Height", 40, 0);
		gd.addCheckbox("Basins (dark) instead of domes",false);

		gd.showDialog();
		if (gd.wasCanceled())
			return DONE;
		domeheight = (int) gd.getNextNumber();
		doIwhite = gd.getNextBoolean ();
		return DOES_8G;
	}

	public void run(ImageProcessor ip) {

		int xe = ip.getWidth();
		int ye = ip.getHeight();
		int x, y, cp, pass=0;
		int [][] pixel = new int [xe][ye];
		int [][] pixel1 = new int [xe][ye];
		int [][] pixel2 = new int [xe][ye];
		double prevsd, currsd, prevmean, currmean;
		ImageStatistics stats;

		//original converted to white particles
		if (doIwhite){
			for(y=0;y<ye;y++) {
				for(x=0;x<xe;x++)
					ip.putPixel(x,y,255-ip.getPixel(x,y));
			}
		}

		ImagePlus i1 = WindowManager.getCurrentImage();

		//create seed
		for(y=0;y<ye;y++) {
			for(x=0;x<xe;x++){
				pixel1[x][y]=ip.getPixel(x,y);//original
				//ip.putPixel(x,y,pixel[x][y]-domeheight);
				pixel2[x][y]=pixel1[x][y]-domeheight;//darkened
			}
		}

		currsd = -1;
		currmean = -1;
		stats=i1.getStatistics();
		prevsd=stats.stdDev;
		prevmean=stats.mean;

		while (currsd!=prevsd && currmean!=prevmean){ //image has changed
			IJ.showStatus("Reconstructing: "+ ++pass);
			for (x=1; x<xe-1; x++) {
				for (y=1; y<ye-1; y++) {
					//white dilate of image except borders
					cp=0;
					if (pixel2[x-1][y-1]>cp) cp=pixel2[x-1][y-1];
					if (pixel2[x  ][y-1]>cp) cp=pixel2[x  ][y-1];
					if (pixel2[x+1][y-1]>cp) cp=pixel2[x+1][y-1];
					if (pixel2[x-1][y  ]>cp) cp=pixel2[x-1][y  ];
					if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
					if (pixel2[x+1][y  ]>cp) cp=pixel2[x+1][y  ];
					if (pixel2[x-1][y+1]>cp) cp=pixel2[x-1][y+1];
					if (pixel2[x  ][y+1]>cp) cp=pixel2[x  ][y+1];
					if (pixel2[x+1][y+1]>cp) cp=pixel2[x+1][y+1];
					pixel[x][y]=cp;
				}
			}

			y=0;
			for (x=1; x<xe-1; x++) {
				//white dilate upper row
				cp=0;
				if (pixel2[x-1][y  ]>cp) cp=pixel2[x-1][y  ];
				if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
				if (pixel2[x+1][y  ]>cp) cp=pixel2[x+1][y  ];
				if (pixel2[x-1][y+1]>cp) cp=pixel2[x-1][y+1];
				if (pixel2[x  ][y+1]>cp) cp=pixel2[x  ][y+1];
				if (pixel2[x+1][y+1]>cp) cp=pixel2[x+1][y+1];
				pixel[x][y]=cp;
			}

			y=ye-1;
			for (x=1; x<xe-1; x++) {
				//white dilate lower row
				cp=0;
				if (pixel2[x-1][y-1]>cp) cp=pixel2[x-1][y-1];
				if (pixel2[x  ][y-1]>cp) cp=pixel2[x  ][y-1];
				if (pixel2[x+1][y-1]>cp) cp=pixel2[x+1][y-1];
				if (pixel2[x-1][y  ]>cp) cp=pixel2[x-1][y  ];
				if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
				if (pixel2[x+1][y  ]>cp) cp=pixel2[x+1][y  ];
				pixel[x][y]=cp;
			}

			x=0;
			for (y=1; y<ye-1; y++) {
				//white dilate left column
				cp=0;
				if (pixel2[x  ][y-1]>cp) cp=pixel2[x  ][y-1];
				if (pixel2[x+1][y-1]>cp) cp=pixel2[x+1][y-1];
				if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
				if (pixel2[x+1][y  ]>cp) cp=pixel2[x+1][y  ];
				if (pixel2[x  ][y+1]>cp) cp=pixel2[x  ][y+1];
				if (pixel2[x+1][y+1]>cp) cp=pixel2[x+1][y+1];
				pixel[x][y]=cp;
			}

			x=xe-1;
			for (y=1; y<ye-1; y++) {
				//white dilate right column
				cp=0;
				if (pixel2[x-1][y-1]>cp) cp=pixel2[x-1][y-1];
				if (pixel2[x  ][y-1]>cp) cp=pixel2[x  ][y-1];
				if (pixel2[x-1][y  ]>cp) cp=pixel2[x-1][y  ];
				if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
				if (pixel2[x-1][y+1]>cp) cp=pixel2[x-1][y+1];
				if (pixel2[x  ][y+1]>cp) cp=pixel2[x  ][y+1];
				pixel[x][y]=cp;
			}

			x=0; //upper left corner
			y=0;
			cp=0;
			if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
			if (pixel2[x+1][y  ]>cp) cp=pixel2[x+1][y  ];
			if (pixel2[x  ][y+1]>cp) cp=pixel2[x  ][y+1];
			if (pixel2[x+1][y+1]>cp) cp=pixel2[x+1][y+1];
			pixel[x][y]=cp;
			x=xe-1; //upper right corner

			//y=0;
			cp=0;
			if (pixel2[x-1][y  ]>cp) cp=pixel2[x-1][y  ];
			if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
			if (pixel2[x-1][y+1]>cp) cp=pixel2[x-1][y+1];
			if (pixel2[x  ][y+1]>cp) cp=pixel2[x  ][y+1];
			pixel[x][y]=cp;

			x=0; //lower left corner
			y=ye-1;
			cp=0;
			if (pixel2[x  ][y-1]>cp) cp=pixel2[x  ][y-1];
			if (pixel2[x+1][y-1]>cp) cp=pixel2[x+1][y-1];
			if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
			if (pixel2[x+1][y  ]>cp) cp=pixel2[x+1][y  ];
			pixel[x][y]=cp;

			x=xe-1; //lower right corner
			y=ye-1;
			cp=0;
			if (pixel2[x-1][y-1]>cp) cp=pixel2[x-1][y-1];
			if (pixel2[x  ][y-1]>cp) cp=pixel2[x  ][y-1];
			if (pixel2[x-1][y  ]>cp) cp=pixel2[x-1][y  ];
			if (pixel2[x  ][y  ]>cp) cp=pixel2[x  ][y  ];
			pixel[x][y]=cp;

			//put result
			for (x=0; x<xe; x++) {
				for (y=0; y<ye; y++) {
					if(pixel[x][y]<pixel1[x][y]){
						pixel2[x][y]=pixel[x][y];
						ip.putPixel(x,y, pixel[x][y]);
					}
					else{
						pixel2[x][y]=pixel1[x][y];
						ip.putPixel(x,y, pixel1[x][y]);
					}
				}
			}

			prevsd=currsd;
			prevmean=currmean;
			stats=i1.getStatistics();
			currsd=stats.stdDev;
			currmean=stats.mean;
		}

		//subtract grey-reconstructed image from original
		for (x=0; x<xe; x++) {
			for (y=0; y<ye; y++) {
				//ip.putPixel(x,y, ip.getPixel(x,y)-pixel2[x][y]);
				ip.putPixel(x,y, pixel1[x][y]-pixel2[x][y]);
			}
		}

		//return to original state
		if (doIwhite){
			for(y=0;y<ye;y++) {
				for(x=0;x<xe;x++)
					ip.putPixel(x,y,255-ip.getPixel(x,y));
			}
		}
	}


	void showAbout() {
		IJ.showMessage("About Domes_...",
		"Domes_ by Gabriel Landini,  G.Landini@bham.ac.uk\n"+
		"ImageJ plugin for extracting domes (bright areas) in an image"+
		"up to a desired height. Uses greyscale reconstruction.\n"+
		"Supports black and white foregrounds.");
	}

}
